#!/bin/bash

# variables
#=============================================
# vesrion of Celestia Origin
CO_VERSION="v.12 (30.10.2020)"

# version of Celestia
CEL_VERSION="1.6.1"

# basic testing path to original Celestia
basepath(){
	BASEDIR="/Applications/Celestia.app/Contents/Resources/CelestiaResources"
}

# path to script
SCRIPTDIR=$(pwd)

# path to installation archives
ARCHPATH="$SCRIPTDIR/.."

# name of core data archive
CORENAME="Celestia Origin v.12 - Core (30.10.2020).7z"

# name of medres data archive
MEDRESNAME="Celestia Origin v.12 - Medres textures (30.10.2020).7z"

# name of hires data archive
HIRESNAME="Celestia Origin v.12 - Hires textures (30.10.2020).7z"

# name of file containing the removing original Celestia's data
backup_list="macos_backup_list.txt"
#=============================================

# functions
#=============================================
# "yes" or "no" (y/n)
answer_yn(){
	shopt -s nocasematch
	while [[ "$ANSWER" != "y" && "$ANSWER" != "n" ]]; do
		read -p '   Incorrect answer, please type again: ' ANSWER
	done
	if [[ "$ANSWER" = "n" ]]; then
		echo "The installation aborted."
		exit; fi
	shopt -u nocasematch
}

# check default paths
pathcheck(){
	echo -e "\nChecking default Celestia\047s installation folder: $BASEDIR..."
	if [ ! -e "$BASEDIR/celestia.cfg" ]; then
		pathset; fi
}

pathset(){
	shopt -s nocasematch
	if [[ "$BASEDIR" == "r" ]]; then
		basepath
		pathcheck; fi	
	shopt -u nocasematch
	if [ ! -e "$BASEDIR/celestia.cfg" ]; then
		echo -e "   There is no Celestia\047s installation in the specified folder. Please check the path to it."
		read -p 'Try another path? (y/n) ' ANSWER
		answer_yn
		pathtype; fi
}

# set path manually or search again
pathtype(){
	read -p "Please specify the path to Celestia's installation (enter it manually or copy and paste); r - search again: " BASEDIR
	pathset
}

# "continue" or "another path" (c/a)
answer_ca(){
	shopt -s nocasematch
	while [[ "$ANSWER" != "c" && "$ANSWER" != "a" ]]; do
		read -p '   Incorrect answer, please type again: ' ANSWER
		answer_ca; done
	if [[ "$ANSWER" = "a" ]]; then
		pathtype
		next; fi
	shopt -u nocasematch
}

next(){
	echo -e "   Celestia\047s installation was found in $BASEDIR."
	read -p "c - continue, a - select another path: " ANSWER
	answer_ca
}

# search necessary installation archives
arch_search(){
# searching core installation archive
	while [ ! -e "$ARCHPATH/$CORENAME" ]; do
		archpath_0
		echo "File \"$CORENAME\" not found."
		archpath_specify; done
# searching medres installation archive
	if [[ "$INSTALL_MEDRES" == "1" ]]; then
		if [ ! -e "$ARCHPATH/$MEDRESNAME" ]; then
			echo "File \"$MEDRESNAME\" not found."
			echo "Please place it to $ARCHPATH and restart the installation."
			exit; fi; fi
# searching hires installation archive
	if [[ "$INSTALL_HIRES" == "1" ]]; then
		if [ ! -e "$ARCHPATH/$HIRESNAME" ]; then
			echo "File \"$HIRESNAME\" not found."
			echo "Please place it to $ARCHPATH and restart the installation."
			exit; fi; fi
}

archpath_0(){
	if [[ "$ARCHPATH" == "0" ]]; then
		echo "The installation aborted."
		exit; fi
}

archpath_specify(){
	read -p "Please specify the path to it without filename (enter it manually or copy and paste); 0 - exit: " ARCHPATH
}


core_installation(){
	# backup making
	#=============================================
	# create backup directory
	BACKUPDIR="$BASEDIR/backup"
	if [ ! -e "$BACKUPDIR" ]; then
		mkdir "$BACKUPDIR"
	else
		echo -e "\nThere is the backup folder already. The installation can\047t be performed.\n"
		echo "The installation aborted."; exit;	fi
	
	echo "Creating a backup copy..."

	# remove extras/update if exists
	rm -Rf "$BASEDIR/extras/update"

	# backup necessery data
	UPPERDIR="$(dirname "$BASEDIR")"
	while IFS= read -r line ; do
		if [ -e "$UPPERDIR/$line" ]; then
			if [ -f "$UPPERDIR/$line" ]; then
				mkdir -p "$(dirname "$BACKUPDIR/$line")" && cp "$UPPERDIR/$line" "$BACKUPDIR/$line"
			fi
			if [ -d "$UPPERDIR/$line" ]; then
				mkdir -p "$(dirname "$BACKUPDIR/$line")" && cp -r "$UPPERDIR/$line" "$BACKUPDIR/$line"
			fi
		else
			echo -e "   /$line in Celestia\047s installation doesn\047t exist, skipping..."
		fi
	done < "$SCRIPTDIR/lists/$backup_list"

	echo "The backup data was placed to $BACKUPDIR"
	#=============================================
	
	# remove original files
	#=============================================
	# remove original configuration file
	rm -f "$BASEDIR/celestia.cfg"

	# remove necessery data
	while IFS= read -r line ; do
		if [ -e "$UPPERDIR/$line" ]; then
			if [ -f "$UPPERDIR/$line" ]; then
				rm "$UPPERDIR/$line"
			fi
			if [ -d "$UPPERDIR/$line" ]; then
				rm -Rf "$UPPERDIR/$line"
			fi
		fi
	done < "$SCRIPTDIR/lists/$backup_list"

	echo "The necessary items were removed."
	#=============================================

	# extract files
	#=============================================
	# core installation
	echo -e "\nInstalling Celestia Origin\047s core..."
	mv "$UPPERDIR/CelestiaResources" "$UPPERDIR/Celestia"
	chmod +x "$SCRIPTDIR/modules/7zr_macos"
	"$SCRIPTDIR/modules/7zr_macos" x -o"$UPPERDIR" "$ARCHPATH/$CORENAME" Celestia -xr-@"$LORES_EXCLUDE"
	mv "$UPPERDIR/Celestia" "$UPPERDIR/CelestiaResources"
	mv "$BASEDIR/splash.png" "$UPPERDIR"
	mv "$BASEDIR/locale/LC_MESSAGES/po.strings" "$UPPERDIR/ru.lproj/"
	mv "$BASEDIR/locale/LC_MESSAGES/celestia_constellations.strings" "$UPPERDIR/ru.lproj/"
	
	if [ -e "$BASEDIR/textures/lores/2k" ]; then
		cd "$BASEDIR/textures/lores/2k/"
		for f in *; do mv "$f" "$BASEDIR/textures/lores/"; done
		rm -d "$BASEDIR/textures/lores/2k"
	fi

	if [ -e "$BASEDIR/textures/lores/4k" ]; then
		cd "$BASEDIR/textures/lores/4k/"
		for f in *; do mv "$f" "$BASEDIR/textures/lores/"; done
		rm -d "$BASEDIR/textures/lores/4k"
	fi

	echo -e "\nThe core installation completed.\n"
}

eightk_models(){
	read -p "Do you also want to install 8k monolithic textures for models? WARNING: NOT FOR WEAK VIDEO CARDS! (y/n) " ANSWER
	shopt -s nocasematch
	if [[ "$ANSWER" = "y" ]]; then
		install_eightk_models=1; fi
	if [[ "$ANSWER" = "n" ]]; then
		install_eightk_models=0; fi
	while [[ "$ANSWER" != "y" && "$ANSWER" != "n" ]]; do
		read -p '   Incorrect answer, please type again: ' ANSWER
	done
	shopt -u nocasematch
	echo
}

medres_installation(){
	# searching medres installation archive
	while [ ! -e "$ARCHPATH/$MEDRESNAME" ];
	do
		echo "File \"$MEDRESNAME\" not found."
		read -p "Please specify the path to it without filename (enter it manually or copy and paste): " ARCHPATH
	done
	# installation
	echo -e "\nInstalling Celestia Origin\047s medres data..."
	mv "$UPPERDIR/CelestiaResources" "$UPPERDIR/Celestia"

	if [[ "$MEDRES_EXCLUDE" == "" ]]; then
		"$SCRIPTDIR/modules/7zr_macos" x -o"$UPPERDIR" "$ARCHPATH/$MEDRESNAME" Celestia
	else
		"$SCRIPTDIR/modules/7zr_macos" x -o"$UPPERDIR" "$ARCHPATH/$MEDRESNAME" Celestia -xr-@"$MEDRES_EXCLUDE"; fi	
	mv "$UPPERDIR/Celestia" "$UPPERDIR/CelestiaResources"

	if [ -e "$BASEDIR/textures/medres/16k" ]; then
		cd "$BASEDIR/textures/medres/16k/"
		for f in *; do mv "$f/level4" "$BASEDIR/textures/medres/$f/"; done
		rm -Rf "$BASEDIR/textures/medres/16k/"; fi

	if [[ "$install_eightk_models" == "1" ]]; then
		echo -e "\nInstalling 8k monolithic textures for models..."
		mv "$UPPERDIR/CelestiaResources" "$UPPERDIR/Celestia"
		"$SCRIPTDIR/modules/7zr_macos" x -o"$UPPERDIR" "$ARCHPATH/$MEDRESNAME" Celestia/textures/medres/8k_for_models
		mv "$UPPERDIR/Celestia" "$UPPERDIR/CelestiaResources"
		cd "$BASEDIR/textures/medres/8k_for_models/"
		for f in *; do mv $f "$BASEDIR/textures/medres/"; done
		rm -d "$BASEDIR/textures/medres/8k_for_models/"
	fi

	echo -e "\nThe medres installation completed.\n"
}

hires_installation(){
	# searching hires installation archive
	while [ ! -e "$ARCHPATH/$HIRESNAME" ]; do
		echo "File \"$HIRESNAME\" not found."
		read -p "Please specify the path to it without filename (enter it manually or copy and paste): " ARCHPATH
	done
	# installation
	echo -e "\nInstalling Celestia Origin\047s hires data..."
	mv "$UPPERDIR/CelestiaResources" "$UPPERDIR/Celestia"

	if [[ "$HIRES_EXCLUDE" == "" ]]; then
		"$SCRIPTDIR/modules/7zr_macos" x -o"$UPPERDIR" "$ARCHPATH/$HIRESNAME" Celestia
	else
	"$SCRIPTDIR/modules/7zr_macos" x -o"$UPPERDIR" "$ARCHPATH/$HIRESNAME" Celestia -xr-@"$HIRES_EXCLUDE"; fi
	mv "$UPPERDIR/Celestia" "$UPPERDIR/CelestiaResources"

	if [ -e "$BASEDIR/textures/hires/64k" ]; then
		cd "$BASEDIR/textures/hires/64k/"
		for f in *; do mv "$f/level6" "$BASEDIR/textures/hires/$f/"; done
		rm -Rf "$BASEDIR/textures/hires/64k/"; fi

	echo -e "\nThe hires installation completed.\n"
}

final(){
	echo "The installation completed."
}
#=============================================

echo "This is Celestia Origin $CO_VERSION for Celestia $CEL_VERSION installation script."

read -p "Do you want to continue? (y/n) " ANSWER
answer_yn

# check default paths
basepath
pathcheck

next

# searching installation archives
# set default values (install core only)
INSTALL_MEDRES=0;
INSTALL_HIRES=0;
# searching core installation archive
arch_search

echo -e "\nSelect installation variant (view ReadMe file for more information) or type 0 to exit.\n"

echo -e '1 - "hires-64k"'
echo -e '2 - "hires-32k"\n'

echo -e '3 - "medres-16k"'
echo -e '4 - "medres-8k"\n'

echo -e '5 - "core-4k"'
echo -e '6 - "core-2k"'
echo -e '7 - "core-1k"\n'

read -p "Enter a number between 0 and 7: " N

while [[ "$N" = *[!0-7]* || "$N" = '' || ((${#N} > 1)) ]]; do
	echo Incorrect answer.
	read -p "Enter a number between 0 and 7: " N
done

case "$N" in
"0")
	echo "The installation aborted."; exit;;
"1") 
	echo -e '\nInstalling variant: "hires-64k"\n'
	LORES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-5_lores.txt"
	INSTALL_MEDRES=1; MEDRES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-3_medres.txt"
	INSTALL_HIRES=1; HIRES_EXCLUDE=""
	arch_search
	eightk_models
	core_installation
	medres_installation
	hires_installation
	final;;
"2") 
	echo -e '\nInstalling variant: "hires-32k"\n'
	LORES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-5_lores.txt"
	INSTALL_MEDRES=1; MEDRES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-3_medres.txt"
	INSTALL_HIRES=1; HIRES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/2_hires.txt"
	arch_search
	eightk_models
	core_installation
	medres_installation
	hires_installation
	final;;
"3") 
	echo -e '\nInstalling variant: "medres-16k"\n'
	LORES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-5_lores.txt"
	INSTALL_MEDRES=1; MEDRES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-3_medres.txt"
	arch_search
	eightk_models
	core_installation
	medres_installation
	final;;
"4") 
	echo -e '\nInstalling variant: "medres-8k"\n'
	LORES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-5_lores.txt"
	INSTALL_MEDRES=1; MEDRES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/4_medres.txt"
	arch_search
	eightk_models
	core_installation
	medres_installation
	final;;
"5") 
	echo -e '\nInstalling variant: "core-4k"\n'
	LORES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/1-5_lores.txt"
	core_installation
	arch_search	
	final;;
"6") 
	echo -e '\nInstalling variant: "core-2k"\n'
	LORES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/6_lores.txt"
	core_installation
	final;;
"7") 
	echo -e '\nInstalling variant: "core-1k"\n'
	LORES_EXCLUDE="$SCRIPTDIR/lists/macos_exclude/7_lores.txt"
	core_installation
	final;;
esac

exit
